#include "./quick_settings_grid_power_profiles.h"

#include <adwaita.h>

#include "../../../../services/power_profiles_service/power_profiles_service.h"
#include "./../quick_settings_grid_button.h"
#include "quick_settings_grid_power_profiles_menu.h"

static void on_active_profile_change(
    PowerProfilesService *pps, char *power_profile,
    QuickSettingsGridPowerProfilesButton *self) {
    gtk_label_set_text(self->button.subtitle, power_profile);

    const char *icon = power_profiles_service_profile_to_icon(power_profile);
    gtk_image_set_from_icon_name(self->button.icon, icon);

    // on selection of a new profile simulate a revealer button click which
    // closes the menu.
    if (gtk_revealer_get_child_revealed(self->button.revealer)) {
        g_signal_emit_by_name(G_OBJECT(self->button.reveal_button), "clicked");
    }
}

void quick_settings_grid_power_profiles_button_layout(
    QuickSettingsGridPowerProfilesButton *self) {
    // get power profiles service
    PowerProfilesService *power_profiles_service =
        power_profiles_service_get_global();

    // get active profile for subtitle and icon
    const gchar *active_profile =
        power_profiles_service_get_active_profile(power_profiles_service);

    const char *icon = power_profiles_service_profile_to_icon(active_profile);
    quick_settings_grid_button_init(
        &self->button, QUICK_SETTINGS_BUTTON_PERFORMANCE, "Performance",
        active_profile, icon,
        quick_settings_grid_power_profiles_button_get_menu_widget(self), NULL);

    // attach to active profile changed event
    g_signal_connect(power_profiles_service, "active-profile-changed",
                     G_CALLBACK(on_active_profile_change), self);
}

QuickSettingsGridPowerProfilesButton *
quick_settings_grid_power_profiles_button_init() {
    QuickSettingsGridPowerProfilesButton *self =
        g_malloc0(sizeof(QuickSettingsGridPowerProfilesButton));

    self->menu =
        g_object_new(QUICK_SETTINGS_GRID_POWER_PROFILES_MENU_TYPE, NULL);

    quick_settings_grid_power_profiles_button_layout(self);

    return self;
}

GtkWidget *quick_settings_grid_power_profiles_button_get_menu_widget(
    QuickSettingsGridPowerProfilesButton *self) {
    return GTK_WIDGET(
        quick_settings_grid_power_profiles_menu_get_widget(self->menu));
}

void quick_settings_grid_power_profiles_button_free(
    QuickSettingsGridPowerProfilesButton *self) {
    g_debug(
        "quick_settings_grid_power_profiles.c:qs_grid_power_profiles_button_"
        "free() called");

    // kill signals
    g_signal_handlers_disconnect_by_func(power_profiles_service_get_global(),
                                         on_active_profile_change, self);

    // unref our menu
    g_object_unref(self->menu);
    // unref button's container
    g_object_unref(self->button.container);
    // free ourselves
    g_free(self);
}
